%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% renamemultipledicomdirectories.m. Rename DICOM files in multiple
% directories.
% 
% This script renames *.dcm files in multiple directories that are
% indicated by the user. 
%
% Creation of file names:
% a. BrainVoyager renaming style (only possible and presented when the
% function 'dicominfo()' is available in Matlab):
% BrainVoyager renaming scheme: 
% <description>_<date>-<series>-<volume>-<cumulative slice number>.dcm
% (anonymized. For naming exactly identical to BrainVoyager, change line 86 to:
%   descr = [dcmInfo.PatientsName.FamilyName '^' dcmInfo.PatientsName.GivenName];
% b. Original file name. In this case the basename is the original
% filename and the numbering and extension are just added: 
% <basename>_00001.dcm
% c. User specified filename. The user can enter a new name which is used
% as basename. The numbering and extension are just added: 
% <basename>_00001.dcm
%
% Use of the script: a. load in Matlab b. press 'run'
%
% Notes: 
% - Files with other extensions than *.dcm in the directory are ignored.
% - No reordering of the filenames in the filename array takes place, so
% please check afterwards whether the renaming proceeded in the correct
% order (not relevant in case of BrainVoyager renaming style).
% - When using BrainVoyager renaming style (via dicominfo()) and the 
% header values in the DICOM file are not properly specified, Matlab may 
% generate an error (f.e. 'Attribute X has inconsistent length') and the 
% script will stop. The other renaming schemes can be used for these cases.
%
% Hester Breman, Brain Innovation 2005.
% Checked 25-07-14
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if exist('dicominfo') == 2
    h = msgbox('This script will rename DICOM files in multiple directories. The BrainVoyager naming style is <descr>_<date>-<series>-<acquisitionnr>-<instancenr>.dcm, and the other is <basename>_<filenumber>.dcm', 'Rename DICOM files');
    uiwait(h);
else 
    h = msgbox('This script will rename DICOM files in multiple directories. The naming scheme is <basename>_<filenumber>.dcm', 'Rename DICOM files');
    uiwait(h);
end

nrdirs = cellstr('1');
nrdirsusr = inputdlg('How many directories with DICOM files should be renamed?', 'Enter number of directories', 1, nrdirs);
dcmdirs = str2num(nrdirsusr{1});
dirarray = cell(1,dcmdirs);

for i=1:dcmdirs
    dirarray{i} = uigetdir(pwd,['Please select directory ' num2str(i) ' from ' num2str(dcmdirs) ' containing DICOM files to rename']);
end

orig = false;
bvstyle = false;
usrspec = false;

if exist('dicominfo') == 2
    namingchoice = questdlg('What filenaming system would you like?','Renaming DICOM files','Use BrainVoyager DICOM naming style', 'Use original filename','User specified','Use BrainVoyager DICOM naming style');
else
    namingchoice = questdlg('What filenaming system would you like? (the file number and extension will be added automatically, f.e. <basename>_00001.hdr)','Renaming DICOM files','Use original filename','Use shortened original filename','User specified','Use original filename');
end

if strcmp(namingchoice,'User specified')
    usrspec = true;
    basenameUsr = inputdlg('Please specify the new basename (the file number and extension will be added automatically, f.e. <basename>_00001.hdr )', 'Create new basename',1);
elseif strcmp(namingchoice,'Use BrainVoyager DICOM naming style')
    bvstyle = true;
else 
    orig = true;
end

for j=1:dcmdirs
    dcmSelection = [[dirarray{j} filesep] '*.dcm'];
    disp(dcmSelection)
    files = dir(dcmSelection);
    sz = size(files);
    if (sz(1) > 0)
        frstFile = [dirarray{j} filesep files(1).name];
        dcmInfo = dicominfo(frstFile);
        sizedir = size(files);
        for i=1:sizedir(1)    
            [basename, ext] = strtok(files(i).name,'.');
            if bvstyle        
                try
                    dcmInfo = dicominfo([dirarray{j} filesep files(i).name]);
                    descr = dcmInfo.StudyDescription;
                    acqDate = dcmInfo.StudyDate;
                    seriesNr = num2str(dcmInfo.SeriesNumber, '%04d');
                    acqNr = num2str(dcmInfo.AcquisitionNumber, '%04d');
                    instNr = num2str(dcmInfo.InstanceNumber, '%04d');
                    newname = [dirarray{j} filesep descr '_' acqDate '-' seriesNr '-' acqNr '-' instNr];
                catch
                    newname = [dirarray{j} filesep basename '_' num2str(i,'%05d')];
                end
            elseif usrspec, newname = [dirarray{j} filesep basenameUsr{1} '_' num2str(i,'%05d')];
            else newname = [dirarray{j} filesep basename '_' num2str(i,'%05d')];
            end
            [status,message,messageid] = movefile([dirarray{j} filesep files(i).name], [newname ext]);   
        end
    end
end
